DROP TEMPORARY TABLE IF EXISTS temp_tab;
CREATE TEMPORARY TABLE temp_tab (number UInt64);
EXISTS TEMPORARY TABLE temp_tab;
SHOW CREATE TEMPORARY TABLE temp_tab;
SHOW TEMPORARY TABLES LIKE 'temp_tab';
DROP TEMPORARY TABLE temp_tab;
EXISTS TEMPORARY TABLE temp_tab;