drop table if exists test.prewhere_alias;
CREATE TABLE test.prewhere_alias (a UInt8,  b Int32,  c UInt8 ALIAS a,  d Int64 ALIAS b + 1, e Int32 alias a + b) ENGINE = MergeTree ORDER BY tuple();
insert into test.prewhere_alias values (1, 1);
select '-';
select a from test.prewhere_alias prewhere a = 1;
select b from test.prewhere_alias prewhere a = 1;
select c from test.prewhere_alias prewhere a = 1;
select d from test.prewhere_alias prewhere a = 1;
select '-';
select a from test.prewhere_alias prewhere b = 1;
select b from test.prewhere_alias prewhere b = 1;
select c from test.prewhere_alias prewhere b = 1;
select d from test.prewhere_alias prewhere b = 1;
select '-';
select a from test.prewhere_alias prewhere c = 1;
select b from test.prewhere_alias prewhere c = 1;
select c from test.prewhere_alias prewhere c = 1;
select d from test.prewhere_alias prewhere c = 1;
select '-';
select a from test.prewhere_alias prewhere d = 2;
select b from test.prewhere_alias prewhere d = 2;
select c from test.prewhere_alias prewhere d = 2;
select d from test.prewhere_alias prewhere d = 2;
select '-';
select a from test.prewhere_alias prewhere a;
select b from test.prewhere_alias prewhere a;
select c from test.prewhere_alias prewhere a;
select d from test.prewhere_alias prewhere a;
select '-';
select a from test.prewhere_alias prewhere c;
select b from test.prewhere_alias prewhere c;
select c from test.prewhere_alias prewhere c;
select d from test.prewhere_alias prewhere c;
drop table if exists test.prewhere_alias;

