create table hits_10m_raw (     WatchID BIGINT,     JavaEnable SMALLINT,     Title STRING,     GoodEvent SMALLINT,     EventTime TIMESTAMP,     EventDate TIMESTAMP,     CounterID BIGINT,     ClientIP BIGINT,     RegionID BIGINT,     UserID BIGINT,     CounterClass TINYINT,     OS SMALLINT,     UserAgent SMALLINT,     URL STRING,     Referer STRING,     Refresh TINYINT,     RefererCategoryID INT,     RefererRegionID BIGINT,     URLCategoryID INT,     URLRegionID BIGINT,     ResolutionWidth INT,     ResolutionHeight INT,     ResolutionDepth SMALLINT,     FlashMajor SMALLINT,     FlashMinor SMALLINT,     FlashMinor2 STRING,     NetMajor SMALLINT,     NetMinor SMALLINT,     UserAgentMajor INT,     UserAgentMinor STRING,     CookieEnable SMALLINT,     JavascriptEnable SMALLINT,     IsMobile SMALLINT,     MobilePhone SMALLINT,     MobilePhoneModel STRING,     Params STRING,     IPNetworkID BIGINT,     TraficSourceID SMALLINT,     SearchEngineID INT,     SearchPhrase STRING,     AdvEngineID SMALLINT,     IsArtifical SMALLINT,     WindowClientWidth INT,     WindowClientHeight INT,     ClientTimeZone INT,     ClientEventTime TIMESTAMP,     SilverlightVersion1 SMALLINT,     SilverlightVersion2 SMALLINT,     SilverlightVersion3 BIGINT,     SilverlightVersion4 INT,     PageCharset STRING,     CodeVersion BIGINT,     IsLink SMALLINT,     IsDownload SMALLINT,     IsNotBounce SMALLINT,     FUniqID BIGINT,     OriginalURL STRING,     HID BIGINT,     IsOldCounter SMALLINT,     IsEvent SMALLINT,     IsParameter SMALLINT,     DontCountHits SMALLINT,     WithHash SMALLINT,     HitColor STRING,     LocalEventTime TIMESTAMP,     Age SMALLINT,     Sex SMALLINT,     Income SMALLINT,     Interests INT,     Robotness SMALLINT,     RemoteIP BIGINT,     WindowName INT,     OpenerName INT,     HistoryLength SMALLINT,     BrowserLanguage STRING,     BrowserCountry STRING,     SocialNetwork STRING,     SocialAction STRING,     HTTPError INT,     SendTiming BIGINT,     DNSTiming BIGINT,     ConnectTiming BIGINT,     ResponseStartTiming BIGINT,     ResponseEndTiming BIGINT,     FetchTiming BIGINT,     SocialSourceNetworkID SMALLINT,     SocialSourcePage STRING,     ParamPrice BIGINT,     ParamOrderID STRING,     ParamCurrency STRING,     ParamCurrencyID INT,     OpenstatServiceName STRING,     OpenstatCampaignID STRING,     OpenstatAdID STRING,     OpenstatSourceID STRING,     UTMSource STRING,     UTMMedium STRING,     UTMCampaign STRING,     UTMContent STRING,     UTMTerm STRING,     FromTag STRING,     HasGCLID SMALLINT,     RefererHash BIGINT,     URLHash BIGINT,     CLID BIGINT,      UserIDHash BIGINT ) ROW FORMAT DELIMITED FIELDS TERMINATED BY '\t' STORED AS TEXTFILE;

load data local inpath '/opt/dump/dump_0.3/dump_hits_10m_meshed_utf8.tsv' overwrite into table hits_10m_raw;

create table hits_10m ( WatchID BIGINT, JavaEnable SMALLINT, Title STRING, GoodEvent SMALLINT, EventTime TIMESTAMP, EventDate TIMESTAMP, CounterID BIGINT, ClientIP BIGINT, RegionID BIGINT, UserID BIGINT, CounterClass TINYINT, OS SMALLINT, UserAgent SMALLINT, URL STRING, Referer STRING, Refresh TINYINT, RefererCategoryID INT, RefererRegionID BIGINT, URLCategoryID INT, URLRegionID BIGINT, ResolutionWidth INT, ResolutionHeight INT, ResolutionDepth SMALLINT, FlashMajor SMALLINT, FlashMinor SMALLINT, FlashMinor2 STRING, NetMajor SMALLINT, NetMinor SMALLINT, UserAgentMajor INT, UserAgentMinor STRING, CookieEnable SMALLINT, JavascriptEnable SMALLINT, IsMobile SMALLINT, MobilePhone SMALLINT, MobilePhoneModel STRING, Params STRING, IPNetworkID BIGINT, TraficSourceID SMALLINT, SearchEngineID INT, SearchPhrase STRING, AdvEngineID SMALLINT, IsArtifical SMALLINT, WindowClientWidth INT, WindowClientHeight INT, ClientTimeZone INT, ClientEventTime TIMESTAMP, SilverlightVersion1 SMALLINT, SilverlightVersion2 SMALLINT, SilverlightVersion3 BIGINT, SilverlightVersion4 INT, PageCharset STRING, CodeVersion BIGINT, IsLink SMALLINT, IsDownload SMALLINT, IsNotBounce SMALLINT, FUniqID BIGINT, OriginalURL STRING, HID BIGINT, IsOldCounter SMALLINT, IsEvent SMALLINT, IsParameter SMALLINT, DontCountHits SMALLINT, WithHash SMALLINT, HitColor STRING, LocalEventTime TIMESTAMP, Age SMALLINT, Sex SMALLINT, Income SMALLINT, Interests INT, Robotness SMALLINT, RemoteIP BIGINT, WindowName INT, OpenerName INT, HistoryLength SMALLINT, BrowserLanguage STRING, BrowserCountry STRING, SocialNetwork STRING, SocialAction STRING, HTTPError INT, SendTiming BIGINT, DNSTiming BIGINT, ConnectTiming BIGINT, ResponseStartTiming BIGINT, ResponseEndTiming BIGINT, FetchTiming BIGINT, SocialSourceNetworkID SMALLINT, SocialSourcePage STRING, ParamPrice BIGINT, ParamOrderID STRING, ParamCurrency STRING, ParamCurrencyID INT, OpenstatServiceName STRING, OpenstatCampaignID STRING, OpenstatAdID STRING, OpenstatSourceID STRING, UTMSource STRING, UTMMedium STRING, UTMCampaign STRING, UTMContent STRING, UTMTerm STRING, FromTag STRING, HasGCLID SMALLINT, RefererHash BIGINT, URLHash BIGINT, CLID BIGINT, UserIDHash BIGINT ) CLUSTERED BY (EventDate) SORTED BY(CounterID, EventDate, UserIDHash, EventTime) INTO 10 BUCKETS STORED AS ORC tblproperties("orc.compress"="ZLIB");

insert overwrite table hits_10m select * from hits_10m_raw;

--drop table hits_10m_raw;