#pragma once

#include <common/Types.h>
#include <common/strong_typedef.h>

/** Represents number of days since 1970-01-01.
  * See DateLUTImpl for usage examples.
  */
STRONG_TYPEDEF(UInt16, DayNum)
