SELECT materialize(toFixedString('', 1)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 2)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 3)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 4)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 5)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 6)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 7)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 8)) AS x FROM system.one GROUP BY x;
SELECT materialize(toFixedString('', 9)) AS x FROM system.one GROUP BY x;
