// autogenerated by ./StorageSystemContributors.sh
const char * auto_contributors[] {
    "Alberto",
    "Aleksandra (Ася)",
    "Alex Bocharov",
    "Alex Krash",
    "Alex Zatelepin",
    "Alexander Avdonkin",
    "Alexander Krasheninnikov",
    "Alexander Kuranoff",
    "Alexander Lukin",
    "Alexander Makarov",
    "Alexander Marshalov",
    "Alexander Millin",
    "Alexander Prudaev",
    "Alexander Sapin",
    "Alexandr Krasheninnikov",
    "Alexandr Orlov",
    "Alexei Averchenko",
    "Alexey Arno",
    "Alexey Milovidov",
    "Alexey Tronov",
    "Alexey Vasiliev",
    "Alexey Zatelepin",
    "Alexsey Shestakov",
    "Aliaksandr Pliutau",
    "Amos Bird",
    "Amy Krishnevsky",
    "Anastasiya Tsarkova",
    "AndreevDm",
    "Andrew Grigorev",
    "Andrey",
    "Andrey Dudin",
    "Andrey M",
    "Andrey Mironov",
    "Andrey Urusov",
    "Anton Popov",
    "Anton Tihonov",
    "Anton Tikhonov",
    "Arsen Hakobyan",
    "Artem Andreenko",
    "Artem Zuikov",
    "Artemeey",
    "Artemkin Pavel",
    "Arthur Tokarchuk",
    "Atri Sharma",
    "Babacar Diassé",
    "BayoNet",
    "BlahGeek",
    "Bogdan",
    "Bogdan Voronin",
    "Bolinov",
    "Brett Hoerner",
    "Bulat Gaifullin",
    "Chen Yufei",
    "Ciprian Hacman",
    "Constantin S. Pan",
    "CurtizJ",
    "Daniel Bershatsky",
    "Daniel Dao",
    "Denis Burlaka",
    "Denis Zhuravlev",
    "Derek Perkins",
    "Dmitry Bilunov",
    "Dmitry Galuza",
    "Dmitry Luhtionov",
    "Dmitry Moskowski",
    "Dmitry Petukhov",
    "Dmitry S..ky  / skype: dvska-at-skype",
    "Elghazal Ahmed",
    "Emmanuel Donin de Rosière",
    "Eugene Klimov",
    "Eugene Konkov",
    "Evgeniy Gatov",
    "Evgeniy Udodov",
    "Evgeny Konkov",
    "Flowyi",
    "Fruit of Eden",
    "George",
    "George3d6",
    "Guillaume Tassery",
    "Hamoon",
    "Hiroaki Nakamura",
    "Igor",
    "Igor Hatarist",
    "Igor Strykhar",
    "Ilya",
    "Ilya Breev",
    "Ilya Khomutov",
    "Ilya Korolev",
    "Ilya Shipitsin",
    "Ivan",
    "Ivan Babrou",
    "Ivan Blinkov",
    "Ivan He",
    "Ivan Lezhankin",
    "Ivan Zhukov",
    "Jason",
    "Jean Baptiste Favre",
    "Jonatas Freitas",
    "Keiji Yoshida",
    "Kirill Malev",
    "Kirill Shvakov",
    "Koblikov Mihail",
    "KochetovNicolai",
    "Konstantin Grabar",
    "Konstantin Lebedev",
    "Konstantin Podshumok",
    "Leonardo Cecchi",
    "Lev Borodin",
    "Liu Cong",
    "LiuCong",
    "LiuYangkuan",
    "Luis Bosque",
    "Maks Skorokhod",
    "Maksim",
    "Marek Vavrusa",
    "Marek Vavruša",
    "Marek Vavruša",
    "Marsel Arduanov",
    "Max Akhmedov",
    "Max Vetrov",
    "Maxim Fridental",
    "Maxim Khrisanfov",
    "Maxim Nikulin",
    "MaximAL",
    "Michael Furmur",
    "Michael Kolupaev",
    "Michael Razuvaev",
    "Mikhail Filimonov",
    "Mikhail Salosin",
    "Mikhail Surin",
    "Mikhail f. Shiryaev",
    "Milad Arabi",
    "Narek Galstyan",
    "Nicolae Vartolomei",
    "Nikita Vasilev",
    "Nikolai Kochetov",
    "Nikolay Kirsh",
    "Nikolay Vasiliev",
    "Nikolay Volosatov",
    "Okada Haruki",
    "Oleg Komarov",
    "Oleg Obleukhov",
    "Olga Khvostikova",
    "Orivej Desh",
    "Pavel",
    "Pavel Kartaviy",
    "Pavel Kartavyy",
    "Pavel Litvinenko",
    "Pavel Patrin",
    "Pavel Yakunin",
    "Pavlo Bashynskiy",
    "Pawel Rog",
    "Ravengg",
    "Reto Kromer",
    "Roman Lipovsky",
    "Roman Nozdrin",
    "Roman Peshkurov",
    "Roman Tsisyk",
    "Sabyanin Maxim",
    "SaltTan",
    "Sergei Tsetlin (rekub)",
    "Sergey Elantsev",
    "Sergey Fedorov",
    "Sergey Lazarev",
    "Sergey Magidovich",
    "Sergey V. Galtsev",
    "Sergey Zaikin",
    "Silviu Caragea",
    "Simon Podlipsky",
    "Sjoerd Mulder",
    "Snow",
    "Stanislav Pavlovichev",
    "Stas Pavlovichev",
    "SuperBot",
    "Tangaev",
    "The-Alchemist",
    "Tobias Adamson",
    "Tsarkova Anastasia",
    "Vadim",
    "Vadim Plakhtinskiy",
    "Vadim Skipin",
    "VadimPE",
    "Valera Ryaboshapko",
    "Vasily Nemkov",
    "Vasily Okunev",
    "Veloman Yunkan",
    "Veniamin Gvozdikov",
    "Victor Tarnavsky",
    "Vitaliy Lyudvichenko",
    "Vitaly Baranov",
    "Vitaly Samigullin",
    "Vlad Arkhipov",
    "Vladimir Chebotarev",
    "Vladimir Kolobaev",
    "Vladimir Kozbin",
    "Vladimir Smirnov",
    "Vladislav Rassokhin",
    "Vojtech Splichal",
    "Vsevolod Orlov",
    "Vyacheslav Alipov",
    "William Shallum",
    "Winter Zhang",
    "Yegor Andreenko",
    "Yuri Dyachenko",
    "Yurii Vlasenko",
    "Yury Karpovich",
    "Yury Stankevich",
    "abdrakhmanov",
    "abyss7",
    "alesapin",
    "alexey-milovidov",
    "ap11",
    "aprudaev",
    "artpaul",
    "avsharapov",
    "blazerer",
    "bseng",
    "cekc",
    "champtar",
    "chenxing-xc",
    "chenxing.xc",
    "chertus",
    "daoready",
    "decaseal",
    "egatov",
    "elBroom",
    "ezhaka",
    "f1yegor",
    "felixoid",
    "filimonov",
    "flow",
    "ggerogery",
    "glockbender",
    "hotid",
    "igor",
    "igor.lapko",
    "ivanzhukov",
    "javi",
    "javi santana",
    "kmeaw",
    "ks1322",
    "kshvakov",
    "leozhang",
    "liuyimin",
    "lomberts",
    "mf5137",
    "mfridental",
    "morty",
    "moscas",
    "ns-vasilev",
    "ogorbacheva",
    "orantius",
    "peshkurov",
    "proller",
    "pyos",
    "robot-clickhouse",
    "robot-metrika-test",
    "root",
    "santaux",
    "serebrserg",
    "shedx",
    "stavrolia",
    "sundy-li",
    "sundyli",
    "topvisor",
    "velom",
    "zamulla",
    "zhang2014",
    "Георгий Кондратьев",
    "Дмитрий Канатников",
    "Иванов Евгений",
    "Павел Литвиненко",
    "Смитюх Вячеслав",
    "Сундуков Алексей",
    "张健",
    "谢磊",
    nullptr};
