create table hits_10m(    WatchID BIGINT,    JavaEnable TINYINT UNSIGNED,    Title VARCHAR(1024),    GoodEvent SMALLINT,    EventTime DATETIME,    EventDate DATE,    CounterID INTEGER UNSIGNED,    ClientIP INTEGER UNSIGNED,    RegionID INTEGER UNSIGNED,    UserID BIGINT,    CounterClass TINYINT,    OS SMALLINT,    UserAgent SMALLINT,    URL VARCHAR(6072),    Referer VARCHAR(2048),    Refresh TINYINT,    RefererCategoryID SMALLINT UNSIGNED,    RefererRegionID INTEGER UNSIGNED,    URLCategoryID SMALLINT UNSIGNED,    URLRegionID INTEGER UNSIGNED,    ResolutionWidth SMALLINT UNSIGNED,    ResolutionHeight SMALLINT UNSIGNED,    ResolutionDepth TINYINT UNSIGNED,    FlashMajor TINYINT UNSIGNED,    FlashMinor TINYINT UNSIGNED,    FlashMinor2 VARCHAR(256),    NetMajor TINYINT UNSIGNED,    NetMinor TINYINT UNSIGNED,    UserAgentMajor SMALLINT UNSIGNED,    UserAgentMinor CHAR(2),    CookieEnable TINYINT UNSIGNED,    JavascriptEnable TINYINT UNSIGNED,    IsMobile TINYINT UNSIGNED,    MobilePhone TINYINT UNSIGNED,    MobilePhoneModel VARCHAR(80),    Params VARCHAR(2048),    IPNetworkID INT UNSIGNED,    TraficSourceID SMALLINT,    SearchEngineID SMALLINT UNSIGNED,    SearchPhrase VARCHAR(1024),    AdvEngineID TINYINT UNSIGNED,    IsArtifical TINYINT UNSIGNED,    WindowClientWidth SMALLINT UNSIGNED,    WindowClientHeight SMALLINT UNSIGNED,    ClientTimeZone INTEGER,    ClientEventTime DATETIME,    SilverlightVersion1 TINYINT UNSIGNED,    SilverlightVersion2 TINYINT UNSIGNED,    SilverlightVersion3 INT UNSIGNED,    SilverlightVersion4 SMALLINT UNSIGNED,    PageCharset VARCHAR(80),    CodeVersion INT UNSIGNED,    IsLink TINYINT UNSIGNED,    IsDownload TINYINT UNSIGNED,    IsNotBounce TINYINT UNSIGNED,    FUniqID BIGINT,    OriginalURL VARCHAR(6072),    HID INT UNSIGNED,    IsOldCounter TINYINT UNSIGNED,    IsEvent TINYINT UNSIGNED,    IsParameter TINYINT UNSIGNED,    DontCountHits TINYINT UNSIGNED,    WithHash TINYINT UNSIGNED,    HitColor CHAR(1),    LocalEventTime DATETIME,    Age TINYINT UNSIGNED,    Sex TINYINT UNSIGNED,    Income TINYINT UNSIGNED,    Interests SMALLINT UNSIGNED,    Robotness TINYINT UNSIGNED,    RemoteIP INT UNSIGNED,    WindowName INT,    OpenerName INT,    HistoryLength SMALLINT,    BrowserLanguage CHAR(2),    BrowserCountry CHAR(2),    SocialNetwork VARCHAR(128),    SocialAction VARCHAR(128),    HTTPError SMALLINT UNSIGNED,    SendTiming INT UNSIGNED,    DNSTiming INT UNSIGNED,    ConnectTiming INTEGER UNSIGNED,    ResponseStartTiming INTEGER UNSIGNED,    ResponseEndTiming INTEGER UNSIGNED,    FetchTiming INTEGER UNSIGNED,    SocialSourceNetworkID TINYINT UNSIGNED,    SocialSourcePage VARCHAR(128),    ParamPrice BIGINT,    ParamOrderID VARCHAR(80),    ParamCurrency CHAR(3),    ParamCurrencyID SMALLINT UNSIGNED,    OpenstatServiceName VARCHAR(80),    OpenstatCampaignID VARCHAR(80),    OpenstatAdID VARCHAR(80),    OpenstatSourceID VARCHAR(80),    UTMSource VARCHAR(256),    UTMMedium VARCHAR(256),    UTMCampaign VARCHAR(256),    UTMContent VARCHAR(256),    UTMTerm VARCHAR(256),    FromTag VARCHAR(256),    HasGCLID TINYINT UNSIGNED,    RefererHash BIGINT,    URLHash BIGINT,    CLID INTEGER UNSIGNED,     UserIDHash BIGINT UNSIGNED) ENGINE=MYISAM;

CREATE INDEX hits_10m_ind on hits_10m (CounterID, EventDate, UserIDHash, EventTime) using BTREE;

load data infile '/opt/dump/dump_0.3/dump_hits_10m_meshed_utf8.tsv' into table hits_10m FIELDS TERMINATED BY '\t'   ESCAPED BY '\\' ;


