
SELECT 'Trivial case';

SELECT multiIf(1, 2, 1, 3, 4);
SELECT multiIf(1, 'A', 1, 'BC', 'DEF');
SELECT multiIf(1, toFixedString('A', 16), 1, toFixedString('BC', 16), toFixedString('DEF', 16));
SELECT multiIf(1, [1,2], 1, [3,4], [5,6]);
SELECT multiIf(1, ['A', 'B'], 1, ['C', 'D'], ['E', 'F']);
SELECT multiIf(rand() % 2 = 0, emptyArrayString(), emptyArrayString());
SELECT multiIf(rand() % 2 = 0, emptyArrayUInt8(), emptyArrayUInt8());
SELECT multiIf(rand() % 2 = 0, '', '');

SELECT 'Numeric branches';

SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt8(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt16(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt32(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toInt64(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toUInt64(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt8(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toUInt64(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt16(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toUInt64(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt32(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt8(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt16(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt32(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toUInt64(1), (number % 3) = 0, toUInt64(2), toUInt64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat32(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt8(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt16(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toUInt32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat32(2), toFloat64(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toUInt8(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toUInt16(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toUInt32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toFloat32(3)) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, toFloat64(1), (number % 3) = 0, toFloat64(2), toFloat64(3)) FROM system.numbers LIMIT 10;

SELECT 'Numeric array branches';

SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt8(1), toInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt16(1), toInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt32(1), toInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toInt64(1), toInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt8(1), toUInt8(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt16(1), toUInt16(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toInt64(2), toInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt64(3), toInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt32(1), toUInt32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toUInt64(1), toUInt64(2)], (number % 3) = 0, [toUInt64(2), toUInt64(3)], [toUInt64(3), toUInt64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat32(1), toFloat32(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt8(2), toInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt16(2), toInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toInt32(2), toInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt8(2), toUInt8(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt16(2), toUInt16(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toUInt32(2), toUInt32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat32(2), toFloat32(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt8(3), toInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt16(3), toInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toInt32(3), toInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt8(3), toUInt8(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt16(3), toUInt16(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toUInt32(3), toUInt32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat32(3), toFloat32(3)]) FROM system.numbers LIMIT 10;
SELECT multiIf((number % 2) = 0, [toFloat64(1), toFloat64(2)], (number % 3) = 0, [toFloat64(2), toFloat64(3)], [toFloat64(3), toFloat64(3)]) FROM system.numbers LIMIT 10;

SELECT 'String branches';

DROP TABLE IF EXISTS test.multi_if_check;
CREATE TABLE test.multi_if_check(col1 UInt64, col2 String, col3 String, col4 String) ENGINE=TinyLog;
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(1, 'A', 'AB', 'ABC');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(2, 'B', 'BC', 'BCD');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(3, 'C', 'CD', 'CDE');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(4, 'D', 'DE', 'DEF');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(5, 'E', 'EF', 'EFG');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(6, 'F', 'FG', 'FGH');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(7, 'G', 'GH', 'GHI');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(8, 'H', 'HI', 'HIJ');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(9, 'I', 'IJ', 'IJK');
INSERT INTO test.multi_if_check(col1, col2, col3, col4) VALUES(10, 'J', 'JK', 'JKL');

SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, col2, 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString(col2, 16), 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, toFixedString('foo', 16), 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, 'foo', 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, col2, 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString(col2, 16), 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, toFixedString('foo', 16), 1, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', (col1 % 3) = 0, 'bar', 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, col3, col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, col3, toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, col3, toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, col3, 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString(col3, 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString(col3, 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString(col3, 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString(col3, 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString('bar', 16), col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString('bar', 16), toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString('bar', 16), toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, toFixedString('bar', 16), 'baz') FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, 'bar', col4) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, 'bar', toFixedString(col4, 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, 'bar', toFixedString('baz', 16)) FROM test.multi_if_check;
SELECT multiIf(1, 'foo', 1, 'bar', 'baz') FROM test.multi_if_check;

DROP TABLE IF EXISTS test.multi_if_check;

SELECT 'String array branches';

CREATE TABLE test.multi_if_check(col1 UInt64, col2 String, col3 String, col4 String, col5 String, col6 String, col7 String) ENGINE=TinyLog;
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(1, 'A', 'AB', 'ABC', 'ABCD', 'ABCDE', 'ABCDEF');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(2, 'B', 'BC', 'BCD', 'BCDE', 'BCDEF', 'BCDEFG');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(3, 'C', 'CD', 'CDE', 'CDEF', 'CDEFG', 'CDEFGH');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(4, 'D', 'DE', 'DEF', 'DEFG', 'DEFGH', 'DEFGHI');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(5, 'E', 'EF', 'EFG', 'EFGH', 'EFGHI', 'EFGHIJ');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(6, 'F', 'FG', 'FGH', 'FGHI', 'FGHIJ', 'FGHIJK');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(7, 'G', 'GH', 'GHI', 'GHIJ', 'GHIJK', 'GHIJKL');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(8, 'H', 'HI', 'HIJ', 'HIJK', 'HIJKL', 'HIJKLM');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(9, 'I', 'IJ', 'IJK', 'IJKL', 'IJKLM', 'IJKLMN');
INSERT INTO test.multi_if_check(col1, col2, col3, col4, col5, col6, col7) VALUES(10, 'J', 'JK', 'JKL', 'JKLM', 'JKLMN', 'JKLMNO');

SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, col3], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, [col2, 'bar'], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', col3], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf((col1 % 2) = 0, ['foo', 'bar'], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, col3], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, [col2, 'bar'], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', col3], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], (col1 % 3) = 0, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, [col4, 'bar'], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', col5], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', col5], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', col5], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', col5], ['foo', 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', 'bar'], [col6, col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', 'bar'], [col6, 'bar']) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', 'bar'], ['foo', col7]) FROM test.multi_if_check;
SELECT multiIf(1, ['foo', 'bar'], 1, ['foo', 'bar'], ['foo', 'bar']) FROM test.multi_if_check;

DROP TABLE IF EXISTS test.multi_if_check;

SELECT 'Miscellaneous';

CREATE TABLE test.multi_if_check(col1 UInt64) ENGINE=TinyLog;
INSERT INTO test.multi_if_check(col1) VALUES (11225),(20273),(213),(240),(12),(187),(29252);
INSERT INTO test.multi_if_check(col1) VALUES (1),(65),(208),(9),(154),(20),(191),(2),(66),(970),(56),(144),(49),(10317),(145),(21623),(225);
INSERT INTO test.multi_if_check(col1) VALUES (10658),(11182),(11036),(194),(16),(54),(23),(172),(10748),(52),(63),(11282),(237),(55),(190);
INSERT INTO test.multi_if_check(col1) VALUES (11162),(197),(4),(64),(17),(59),(51),(1091),(14),(13),(10645),(73),(157),(169),(43),(47),(11111);
INSERT INTO test.multi_if_check(col1) VALUES (11002),(99),(42),(142),(40),(20259),(15),(976),(11220),(193),(21),(3),(973),(35),(148),(10277),(10253);
INSERT INTO test.multi_if_check(col1) VALUES (155),(972),(159),(959),(39),(50),(236),(969),(114678),(143),(37),(20167),(20086),(10536),(28948),(10355);
INSERT INTO test.multi_if_check(col1) VALUES (7),(192),(11119),(67),(6),(8),(10841),(141),(10951),(222),(10752),(10363),(10842),(24881),(11287),(198);
INSERT INTO test.multi_if_check(col1) VALUES (11453),(26),(147),(44),(19),(76),(29349),(10987),(28896),(75),(11168),(11084),(62),(46),(10747),(24);
INSERT INTO test.multi_if_check(col1) VALUES (10664),(966),(11256),(53),(10945),(10871),(158),(20544),(20540),(163),(10262),(11080),(0),(11159),(239);
INSERT INTO test.multi_if_check(col1) VALUES (10995),(11),(11143),(11212),(24893),(10318),(68),(21949),(28),(971),(153),(10689),(38),(11474),(11029);
INSERT INTO test.multi_if_check(col1) VALUES (11067),(21636),(965),(10761),(10324),(164),(977),(45),(10),(10944),(964),(20224),(20536),(102880),(33);
INSERT INTO test.multi_if_check(col1) VALUES (11232),(10315),(78),(11217),(10373),(11514),(10343),(48),(22),(20197),(10367),(36),(11116),(195),(10274);
INSERT INTO test.multi_if_check(col1) VALUES (115),(11069),(30),(11266),(10891),(11235),(112529),(206),(10393),(10712),(10649),(11164),(10511),(10295);
INSERT INTO test.multi_if_check(col1) VALUES (11139),(10347),(146),(11079),(961),(11231),(10358),(10653),(11358),(165),(11115),(1095),(960),(10992),(20221);
INSERT INTO test.multi_if_check(col1) VALUES (5),(10716),(102),(974),(10691),(102444),(11391),(10897),(10306),(10298),(10896),(21609),(118),(11148),(11451);
INSERT INTO test.multi_if_check(col1) VALUES (10398),(221),(975),(80),(162),(28051),(10838),(10765),(1058),(11464),(74),(21134),(21422),(10313),(28401),(20539);
INSERT INTO test.multi_if_check(col1) VALUES (10418),(235),(25),(179),(26030),(28381),(11091),(27398),(11108),(968),(10300),(11469),(35393),(10733),(11283),(11202);

SELECT DISTINCT col1, multiIf(col1 != 213, 'Москва', 'Мир') AS k FROM test.multi_if_check LIMIT 10;
DROP TABLE IF EXISTS test.multi_if_check;
